@@ -13,8 +13,10 @@
 # limitations under the License.
 
 import logging
+import os
 
 from .state import AcceleratorState
+from .utils import DistributedType
 
 
 class MultiProcessAdapter(logging.LoggerAdapter):
@@ -28,7 +30,12 @@ class MultiProcessAdapter(logging.LoggerAdapter):
     @staticmethod
     def _should_log(main_process_only):
         "Check if log should be performed"
-        return not main_process_only or (main_process_only and AcceleratorState().local_process_index == 0)
+        state = AcceleratorState()
+        if state.distributed_type != DistributedType.MEGATRON_LM:
+            process_index_flag = state.local_process_index == 0
+        else:
+            process_index_flag = state.process_index == state.num_processes - 1
+        return not main_process_only or (main_process_only and process_index_flag)
 
     def log(self, level, msg, *args, **kwargs):
         """
@@ -43,7 +50,7 @@ class MultiProcessAdapter(logging.LoggerAdapter):
             self.logger.log(level, msg, *args, **kwargs)
 
 
-def get_logger(name: str):
+def get_logger(name: str, log_level: str = None):
     """
     Returns a `logging.Logger` for `name` that can handle multiprocessing.
 
@@ -52,6 +59,8 @@ def get_logger(name: str):
     Args:
         name (`str`):
             The name for the logger, such as `__file__`
+        log_level (`str`, *optional*):
+            The log level to use. If not passed, will default to the `LOG_LEVEL` environment variable, or `INFO` if not
 
     Example:
 
@@ -62,7 +71,15 @@ def get_logger(name: str):
 
     >>> logger.info("My log", main_process_only=False)
     >>> logger.debug("My log", main_process_only=True)
+
+    >>> logger = get_logger(__name__, accelerate_log_level="DEBUG")
+    >>> logger.info("My log")
+    >>> logger.debug("My second log")
     ```
     """
+    if log_level is None:
+        log_level = os.environ.get("ACCELERATE_LOG_LEVEL", None)
     logger = logging.getLogger(name)
+    if log_level is not None:
+        logger.setLevel(log_level.upper())
     return MultiProcessAdapter(logger, {})
