@@ -13,12 +13,18 @@
 # limitations under the License.
 
 import importlib
+import os
 import sys
+import warnings
+from distutils.util import strtobool
 from functools import lru_cache
 
 import torch
 
-from .versions import is_torch_version
+from packaging.version import parse
+
+from .environment import parse_flag_from_env
+from .versions import compare_versions, is_torch_version
 
 
 # The package importlib_metadata is in a different place, depending on the Python version.
@@ -87,6 +93,19 @@ def is_bf16_available(ignore_tpu=False):
     return False
 
 
+def is_megatron_lm_available():
+    if strtobool(os.environ.get("ACCELERATE_USE_MEGATRON_LM", "False")) == 1:
+        package_exists = importlib.util.find_spec("megatron") is not None
+        if package_exists:
+            megatron_version = parse(importlib_metadata.version("megatron-lm"))
+            return compare_versions(megatron_version, ">=", "2.2.0")
+    return False
+
+
+def is_safetensors_available():
+    return importlib.util.find_spec("safetensors") is not None
+
+
 def is_transformers_available():
     return importlib.util.find_spec("transformers") is not None
 
@@ -116,7 +135,15 @@ def is_boto3_available():
 
 
 def is_rich_available():
-    return importlib.util.find_spec("rich") is not None
+    if importlib.util.find_spec("rich") is not None:
+        if parse_flag_from_env("DISABLE_RICH"):
+            warnings.warn(
+                "The `DISABLE_RICH` flag is deprecated and will be removed in version 0.17.0 of 🤗 Accelerate. Use `ACCELERATE_DISABLE_RICH` instead.",
+                FutureWarning,
+            )
+            return not parse_flag_from_env("DISABLE_RICH")
+        return not parse_flag_from_env("ACCELERATE_DISABLE_RICH")
+    return False
 
 
 def is_sagemaker_available():
@@ -125,3 +152,7 @@ def is_sagemaker_available():
 
 def is_tqdm_available():
     return importlib.util.find_spec("tqdm") is not None
+
+
+def is_mlflow_available():
+    return importlib.util.find_spec("mlflow") is not None
