@@ -16,7 +16,7 @@
 A set of basic tensor ops compatible with tpu, gpu, and multigpu
 """
 
-
+import pickle
 from functools import update_wrapper
 from typing import Any, Mapping
 
@@ -24,6 +24,7 @@ import torch
 from torch.distributed import ReduceOp
 
 from ..state import AcceleratorState
+from .constants import CUDA_DISTRIBUTED_TYPES
 from .dataclasses import DistributedType, TensorInformation
 from .imports import is_tpu_available
 from .versions import is_torch_version
@@ -43,7 +44,7 @@ def is_tensor_information(tensor_info):
 
 def honor_type(obj, generator):
     """
-    Cast a generator to the same type as obj (list, tuple or namedtuple)
+    Cast a generator to the same type as obj (list, tuple, or namedtuple)
     """
     try:
         return type(obj)(generator)
@@ -103,7 +104,7 @@ def recursively_apply(func, data, *args, test_type=is_torch_tensor, error_on_oth
     return data
 
 
-def send_to_device(tensor, device):
+def send_to_device(tensor, device, non_blocking=False):
     """
     Recursively sends the elements in a nested list/tuple/dictionary of tensors to a given device.
 
@@ -117,13 +118,16 @@ def send_to_device(tensor, device):
         The same data structure as `tensor` with all tensors sent to the proper device.
     """
 
-    def _send_to_device(t, device):
-        return t.to(device)
+    def _send_to_device(t, device, non_blocking):
+        try:
+            return t.to(device, non_blocking=non_blocking)
+        except TypeError:  # .to() doesn't accept non_blocking as kwarg
+            return t.to(device)
 
     def _has_to_method(t):
         return hasattr(t, "to")
 
-    return recursively_apply(_send_to_device, tensor, device, test_type=_has_to_method)
+    return recursively_apply(_send_to_device, tensor, device, non_blocking, test_type=_has_to_method)
 
 
 def get_data_structure(data):
@@ -217,11 +221,7 @@ def gather(tensor):
     """
     if AcceleratorState().distributed_type == DistributedType.TPU:
         return _tpu_gather(tensor, name="accelerate.utils.gather")
-    elif AcceleratorState().distributed_type in [
-        DistributedType.DEEPSPEED,
-        DistributedType.MULTI_GPU,
-        DistributedType.FSDP,
-    ]:
+    elif AcceleratorState().distributed_type in CUDA_DISTRIBUTED_TYPES:
         return _gpu_gather(tensor)
     elif AcceleratorState().distributed_type == DistributedType.MULTI_CPU:
         return _cpu_gather(tensor)
@@ -254,11 +254,7 @@ def gather_object(object: Any):
     """
     if AcceleratorState().distributed_type == DistributedType.TPU:
         raise NotImplementedError("gather objects in TPU is not supported")
-    elif AcceleratorState().distributed_type in [
-        DistributedType.DEEPSPEED,
-        DistributedType.MULTI_GPU,
-        DistributedType.FSDP,
-    ]:
+    elif AcceleratorState().distributed_type in CUDA_DISTRIBUTED_TYPES:
         return _gpu_gather_object(object)
     elif AcceleratorState().distributed_type == DistributedType.MULTI_CPU:
         return _cpu_gather_object(object)
@@ -297,11 +293,7 @@ def broadcast(tensor, from_process: int = 0):
     """
     if AcceleratorState().distributed_type == DistributedType.TPU:
         return _tpu_broadcast(tensor, src=from_process, name="accelerate.utils.broadcast")
-    elif AcceleratorState().distributed_type in [
-        DistributedType.DEEPSPEED,
-        DistributedType.MULTI_GPU,
-        DistributedType.FSDP,
-    ]:
+    elif AcceleratorState().distributed_type in CUDA_DISTRIBUTED_TYPES:
         return _gpu_broadcast(tensor, src=from_process)
     elif AcceleratorState().distributed_type == DistributedType.MULTI_CPU:
         return _gpu_broadcast(tensor, src=from_process)
@@ -325,11 +317,7 @@ def broadcast_object_list(object_list, from_process: int = 0):
     if AcceleratorState().distributed_type == DistributedType.TPU:
         for i, obj in enumerate(object_list):
             object_list[i] = xm.mesh_reduce("accelerate.utils.broadcast_object_list", obj, lambda x: x[from_process])
-    elif AcceleratorState().distributed_type in [
-        DistributedType.DEEPSPEED,
-        DistributedType.MULTI_GPU,
-        DistributedType.FSDP,
-    ]:
+    elif AcceleratorState().distributed_type in CUDA_DISTRIBUTED_TYPES:
         torch.distributed.broadcast_object_list(object_list, src=from_process)
     elif AcceleratorState().distributed_type == DistributedType.MULTI_CPU:
         torch.distributed.broadcast_object_list(object_list, src=from_process)
@@ -445,11 +433,7 @@ def reduce(tensor, reduction="mean"):
         if state.distributed_type == DistributedType.TPU:
             xm.all_reduce("sum", cloned_tensor)
             return cloned_tensor
-        elif state.distributed_type in [
-            DistributedType.DEEPSPEED,
-            DistributedType.MULTI_GPU,
-            DistributedType.FSDP,
-        ]:
+        elif state.distributed_type.value in CUDA_DISTRIBUTED_TYPES:
             torch.distributed.all_reduce(cloned_tensor, ReduceOp.SUM)
             return cloned_tensor
         else:
@@ -489,8 +473,6 @@ class ConvertOutputsToFp32:
     Decorator to apply to a function outputing tensors (like a model forward pass) that ensures the outputs in FP16
     precision will be convert back to FP32.
 
-    Use a class instead of a decorator because otherwise, the prepared model can no longer be pickled (issue #273).
-
     Args:
         model_forward (`Callable`):
             The function which outputs we want to treat.
@@ -506,6 +488,11 @@ class ConvertOutputsToFp32:
     def __call__(self, *args, **kwargs):
         return convert_to_fp32(self.model_forward(*args, **kwargs))
 
+    def __getstate__(self):
+        raise pickle.PicklingError(
+            "Cannot pickle a prepared model with automatic mixed precision, please unwrap the model with `Accelerator.unwrap_model(model)` before pickling it."
+        )
+
 
 convert_outputs_to_fp32 = ConvertOutputsToFp32
 
